/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: $
 */

#include "aladder.h"

static int bri_map[21][256];

void
ImageInit()
{
  int i, j;

  for (i = 0; i <= 20; i++) {  
    for (j = 0; j < 256; j++) {
      bri_map[i][j] = j * i / 20;
    }
  }
}

Image *
ImageCreate(int width, int height)
{
  Image *img;

  img = (Image *)malloc(sizeof(Image));
  img->width = width;
  img->height = height;
  img->rgbmap = (u_char *)calloc(width * height * 3, sizeof(u_char));

  return img;
}

void
ImageDestroy(Image *image)
{
  free(image->rgbmap);
  free(image);
}

void
ImageLoad(Image *image, const char *name)
{
  if (image->rgbmap != NULL) {
    free(image->rgbmap);
  }
  get_image(alCore.cg_arc, name, image);
}

void
ImageFill(Image *image, int x, int y, int w, int h, u_char r, u_char g, u_char b)
{
  int i, j, idx;

  for (i = 0; i < h; i++) {
    idx = (image->width * (y + i) + x) * 3;
    for (j = 0; j < w; j++) {
      image->rgbmap[idx++] = r;
      image->rgbmap[idx++] = g;
      image->rgbmap[idx++] = b;
    }
  }
}

void
ImageDisplay(int brightness)
{
  if (brightness == 20) {
    memcpy(alCore.vram->rgbmap, alCore.work->rgbmap, WIDTH * HEIGHT * 3);
  } else {
    int i;
    for (i = 0; i < WIDTH * HEIGHT * 3; i++) {
      alCore.vram->rgbmap[i] = bri_map[brightness][alCore.work->rgbmap[i]];
    }
  }
}
